********************************************************************************
** Digitalization and Inclusive Growth *****************************************
** Income and regional group classification*************************************
********************************************************************************

clear all 
set more off

** Income group classification *************************************************

import excel using "$input_data\\OGHIST.xlsx", sheet(Country Analytical History) cellrange(A12:AM229) 
***OGHIST.xlsx is the FY25 "historical classification by income in XLSX format" available here: https://datahelpdesk.worldbank.org/knowledgebase/articles/906519-world-bank-country-and-lending-groups***

*keep relevant columns
keep A B P AL 

rename A code
rename B economy
rename P inc_grp_2000
rename AL inc_grp_2022

sort economy code

*Compile income group classification variable for years 2000, and 2022
foreach year in 2000 2022 {
	replace inc_grp_`year' = "LIC" if inc_grp_`year' == "L"
	replace inc_grp_`year' = "LMC" if inc_grp_`year' == "LM"
	replace inc_grp_`year' = "UMC" if inc_grp_`year' == "UM"
	replace inc_grp_`year' = "HIC" if inc_grp_`year' == "H"
	replace inc_grp_`year' = "" if inc_grp_`year' == ".."
	label var inc_grp_`year' "World Bank income classification `year'"
}

label var code "ISO code"
label var economy "World Bank economy name"

*Replace the following country names with their WBG official names
replace economy = "Faroe Islands" if economy == "Faeroe Islands"
replace economy = "Korea, Dem. People's Rep." if economy == "Korea, Dem. Rep."

local rsq = ustrunescape("\u2019")
replace economy = "Côte d`rsq'Ivoire" if economy == "Côte d'Ivoire"

save "$output_data\\income_groups.dta", replace