********************************************************************************
** Digitalization and Inclusive Growth *****************************************
** Figure 2 ********************************************************************
********************************************************************************

clear all
set more off

** Graph schemes
set scheme s2color
graph set window fontface Arial


** Load GDP data (downloaded August 2024)
wbopendata, indicator(ny.gdp.pcap.pp.kd; it.net.user.zs; it.cel.sets.p2) clear long

rename countrycode code
keep code countryname year ny_gdp_pcap_pp_kd it_net_user_zs it_cel_sets_p2 
gen lgdppc = log(ny_gdp_pcap_pp_kd)
gen year_str = string(year)


** Merge with income groups (dataset created in "Data_creation_country_groups.do"-file)
merge m:m code using "$output_data\\income_groups.dta"
keep if _merge==3
drop _merge


** Figure 2A: Internet usage ***************************************************
preserve

encode code, gen(icode)
xtset icode year

keep code countryname year lgdppc it_net_user_zs 
keep if year==2000 | year==2021
reshape wide lgdppc it_net_user_zs, i(code countryname) j(year)

twoway scatter it_net_user_zs2000 lgdppc2000, mcolor("0 173 228") ||scatter it_net_user_zs2021 lgdppc2000, mcolor("0 35 69") graphregion(color(white)) bgcolor(white) ylab(0(20)100,nogrid labsize(medlarge)) xlab(6(1)12,labsize(medlarge)) ytitle("Percent of population using the Internet",size(medlarge)) xtitle("Log real GDP per capita, 2000", size(medlarge)) legend(pos(6) col(2) label(1 "Year 2000") label(2 "Year 2021") size(medlarge) region(lstyle(none)))

	graph export  "$output_figure\\Figure2A_Internet_GDPpc_2000-21.pdf", replace
	graph export  "$output_figure\\Figure2A_Internet_GDPpc_2000-21.emf", replace

restore


** Figure 2B: Mobile phone subscriptions ***************************************
preserve

encode code, gen(icode)
xtset icode year

keep code countryname year lgdppc it_cel_sets_p2
keep if year==2000 | year==2021
reshape wide lgdppc it_cel_sets_p2, i(code countryname) j(year)

sum it_cel_sets_p22021, d
local p99 = r(p99)
replace it_cel_sets_p22021 = . if it_cel_sets_p22021 > `p99' & it_cel_sets_p22021!=.

twoway scatter it_cel_sets_p22000 lgdppc2000, mcolor("0 173 228") ||scatter it_cel_sets_p22021 lgdppc2000, mcolor("0 35 69") graphregion(color(white)) bgcolor(white) ylab(0(50)200,nogrid labsize(medlarge)) xlab(6(1)12,labsize(medlarge)) ytitle("Subscriptions per 100 people",size(medlarge)) xtitle("Log real GDP per capita, 2000", size(medlarge)) legend(pos(6) col(2) label(1 "Year 2000") label(2 "Year 2021") size(medlarge) region(lstyle(none)))

	graph export  "$output_figure\\Figure2B_Mobphone_GDPpc_2000-21.pdf", replace
	graph export  "$output_figure\\Figure2B_Mobphone_GDPpc_2000-21.emf", replace

restore

