********************************************************************************
** Digitalization and Inclusive Growth *****************************************
** Figure 3 ********************************************************************
********************************************************************************

clear all
set more off

** Graph schemes
set scheme s2color
graph set window fontface Arial

** Load GDP data (downloaded August 2024)
wbopendata, indicator(ny.gdp.pcap.pp.kd) clear long

rename countrycode code
keep code countryname year ny_gdp_pcap_pp_kd
gen lgdppc = log(ny_gdp_pcap_pp_kd)
gen year_str = string(year)
rename countryname economy

save "$output_data/gdppc.dta", replace


** Figure 3A. Firms' website use ***********************************************

import excel using "$input_data\\WBES_CustomQuery.xlsx", firstrow clear
** World Bank Enterprise Survey: : https://www.enterprisesurveys.org/en/custom-query

rename Economy economy
rename Year year
rename Percentoffirmshavingtheirow website
destring year, replace

replace economy = "Côte d'Ivoire" if economy == "Cote d'Ivoire"
local lsq = ustrunescape("\u2018")
local rsq = ustrunescape("\u2019")
replace economy = "Côte d`rsq'Ivoire" if economy == "Côte d'Ivoire"
replace economy = "Guyana" if economy == "Guyana, CR"
replace economy = "Venezuela, RB" if economy == "Venezuela, R.B."
replace economy = "Türkiye" if economy == "Turkiye"

merge m:1 economy year using "$output_data\\gdppc"
keep if _merge==3
drop _merge

merge m:1 economy using "$output_data\\income_groups"
keep if _merge==3
drop _merge

gen inc_grp_plot = inc_grp_2022
replace inc_grp_plot="LICs and MICs" if inc_grp_plot=="LMC"|inc_grp_plot=="UMC"|inc_grp_plot=="LIC"
replace inc_grp_plot="HICs" if inc_grp_plot=="HIC"
drop if inc_grp_plot == "" 

destring(year), replace
destring(website), replace

gen period_dummy = .
replace period_dummy = 1 if year >= 2006 & year <= 2009
replace period_dummy = 2 if year >= 2019 & year <= 2023
label define period_dummy 1 "2006-09" 2 "2019-23"

set scheme s2color
graph set window fontface Arial


** Scatterplot for 2019-2023 using all available economies (57 LICs and MICs, 34 HICs)

preserve
drop if year <2019
sort economy year
by economy: gen duplicates = cond(_N==1, 0, _n)
drop if duplicates == 1 //if an economy has two obs in 2019-2023, drop the earlier obs (15 dropped)
drop duplicates
encode inc_grp_plot, generate (inc_grp_plot_)

twoway scatter website lgdppc, colorvar(inc_grp_plot_) colordiscrete colorlist("0 173 228" "0 35 69") zlabel(, valuelabel) coloruseplegend plegend(pos(6) col(2) order(1 2)  label(1 "Developing economies") label(2 "Advanced economies") region(lstyle(none)) size(medlarge)) graphregion(color(white)) bgcolor(white) ylab(0(20)100,nogrid labsize(medlarge)) xlab(6(1)12,labsize(medlarge)) ytitle("Percent of firms with own website," "2019-23",size(medlarge)) xtitle("Log real GDP per capita", size(medlarge))

	graph export  "$output_figure\\Figure3A_Firms_with_website_GDPpc_2019-23.pdf", replace
	graph export  "$output_figure\\Figure3A_Firms_with_website_GDPpc_2019-23.emf", replace
	
restore



*****Figure 3B. GovTech maturity ***********************************************

** Graph schemes
set scheme s2color
graph set window fontface Arial

**Process the raw data
import excel "$input_data\\WBG_GovTech Dataset_Oct2022.xlsx", sheet("CG_GTMI_Groups") cellrange(A1:E199) firstrow clear
ren Economy economy
ren Code code
ren Year year
drop Grp
label var GTMI "World Bank Gov Maturity Index"
save "$output_data\\WBG_GovTech_2022.dta", replace

**Make the scatterplot
use "$output_data\\WBG_GovTech_2022.dta", replace
merge 1:1 code year using "$output_data/gdppc.dta"
keep if _merge==3
drop _merge

// Load income group data
merge 1:1 code using "$output_data\\income_groups"
keep if _merge==3
drop _merge

// Generate consolidated developing country income group
rename inc_grp_2022 Incomegroup
replace Incomegroup="LICs and MICs" if Incomegroup=="LMC"|Incomegroup=="UMC"|Incomegroup=="LIC"
replace Incomegroup="HICs" if Incomegroup=="HIC"

gen order=.
replace order=1 if Incomegroup=="LICs and MICs"
replace order=2 if Incomegroup=="HICs"

encode Incomegroup, generate (Incomegroup_)

// Scatterplot 
twoway scatter GTMI lgdppc, colorvar(Incomegroup_) colordiscrete colorlist("0 173 228" "0 35 69") zlabel(, valuelabel) coloruseplegend plegend(pos(6) col(2) order(1 2) label(1 "Developing economies") label(2 "Advanced economies") region(lstyle(none)) size(medlarge)) graphregion(color(white)) bgcolor(white) ylab(0(0.2)1,nogrid labsize(medlarge)) xlab(6(1)12,labsize(medlarge)) ytitle("World Bank GovTech Maturity Index 2022",size(medlarge)) xtitle("Log real GDP per capita", size(medlarge))

	graph export  "$output_figure\\Figure3B_WB GMTI_2022.pdf", replace
	graph export  "$output_figure\\Figure3B_WB GMTI_2022.emf", replace
